﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Incidencia
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Windows;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(IncidenciaViewModel))]
    public class IncidenciaViewModel : Screen, IDataErrorInfo, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _formErrors;
        private string _lastError = "Formulari incomplert o amb errors";
        private List<TipusAvariaDTO> _avariesList;
        private List<EstatIncidenciaDTO> _estatsList;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private IncidenciaDTO _incidenciaModel;

        [ImportingConstructor]
        public IncidenciaViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;           

            AvariesList = client.GetAllTipusAvaria();
            EstatsList = client.GetAllEstatsIncidencia();
        }

        # region VIEW

        // Accions a efectuar quan la vista ha estat enllaçada al ViewModel
        protected override void OnViewAttached(object view, object context)
        {
            base.OnViewAttached(view, context);

            // Comprova si s'ha de fer servir el caché del model.
            if (CacheMain.IsNewIncidencia)
            {
                IncidenciaModel = new IncidenciaDTO() {Id_incidencia = 0, 
                                                       Id_supervisor = CacheMain.CurrentUserId,
                                                       Comentari = "Sense comentaris",
                                                       DataCreacio = DateTime.Now};
                CacheMain.IsNewIncidencia = false;
                CacheMain.CurrentIncidencia = IncidenciaModel;
            }
            else
            {
                IncidenciaModel = CacheMain.CurrentIncidencia;
            }
        }

        // Accions a efectuar quan es surt de la vista
        protected override void OnDeactivate(bool close)
        {
            CacheMain.CurrentIncidencia = IncidenciaModel;
            base.OnDeactivate(close);
        }

        # endregion

        # region Error
        public string FormErrors
        {
            get { return _formErrors; }
            set
            {
                _formErrors = value;
                NotifyOfPropertyChange(() => FormErrors);
            }
        }

        public string LastError
        {
            get { return _lastError; }
            set
            {
                _lastError = value;
                NotifyOfPropertyChange(() => LastError);
            }
        }
        # endregion

        # region Properties

        public IncidenciaDTO IncidenciaModel
        {
            get { return _incidenciaModel; }
            set
            {
                _incidenciaModel = value;
                NotifyOfPropertyChange(() => IncidenciaModel);
            }
        }

        public List<TipusAvariaDTO> AvariesList
        {
            get
            {
                return _avariesList;
            }
            set
            {
                _avariesList = value;
                NotifyOfPropertyChange(() => AvariesList);
            }
        }

        public List<EstatIncidenciaDTO> EstatsList
        {
            get
            {
                return _estatsList;
            }
            set
            {
                _estatsList = value;
                NotifyOfPropertyChange(() => EstatsList);
            }
        }

        /// <summary>
        /// Identificació de l'Incidència a la BBDD
        /// </summary>
        public int Id
        {
            get { return IncidenciaModel.Id_incidencia; }
            set
            {
                IncidenciaModel.Id_incidencia = value;
                NotifyOfPropertyChange(() => Id);
                NotifyOfPropertyChange(() => CanSave);

                if (value < 0)
                {
                    LastError = "L'Id és incorrecte";
                    _dataErrors["Id"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Id"))
                        _dataErrors.Remove("Id");
                }
            }
        }

        public int IdElement
        {
            get { return IncidenciaModel.Id_element; }
            set
            {
                IncidenciaModel.Id_element = value;
                NotifyOfPropertyChange(() => IdElement);
                NotifyOfPropertyChange(() => CanSave);

                if (value < 0)
                {
                    LastError = "L'Id de l'Element és incorrecte";
                    _dataErrors["IdElement"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("IdElement"))
                        _dataErrors.Remove("IdElement");
                }
            }
        }

        public DateTime Data
        {
            get { return IncidenciaModel.DataCreacio; }
            set
            {
                IncidenciaModel.DataCreacio = value;
                NotifyOfPropertyChange(() => Data);
                NotifyOfPropertyChange(() => CanSave);

                if (Data == null)
                {
                    LastError = "Data de creació incorrecta";
                    _dataErrors["Data"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Data"))
                        _dataErrors.Remove("Data");
                }
            }
        }

        public int Avaria
        {
            get { return IncidenciaModel.Id_tipusAvaria; }
            set
            {
                IncidenciaModel.Id_tipusAvaria = value;
                NotifyOfPropertyChange(() => Avaria);
                NotifyOfPropertyChange(() => CanSave);

                if (value == 0)
                {
                    LastError = "Es requereix un tipus d'Avaria";
                    _dataErrors["Avaria"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Avaria"))
                        _dataErrors.Remove("Avaria");
                }
            }
        }


        public string Comentari
        {
            get { return IncidenciaModel.Comentari; }
            set
            {
                IncidenciaModel.Comentari = value;
                NotifyOfPropertyChange(() => Comentari);
            }
        }
        
        # endregion

        # region Commands

        public void UpdateById()
        {
            if (Id > 0)
            {
                IncidenciaModel = client.GetIncidenciaById(Id);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        public void New()
        {            
            events.Publish(new CommandEvent { CommandToExecute = "ShowNewIncidencia" });
        }

        public bool CanSave
        {
            get
            {
                bool isValid = !(IdElement < 1 || Data == null || Avaria < 1 );
                return isValid;
            }
        }

        public void Save()
        {
            if (IncidenciaModel.Id_incidencia == 0)
            {
                if (client.AddIncidencia(IncidenciaModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Incidència enregistrada correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error enregistrant l'Incidència");
                }
            }
            else if (IncidenciaModel.Id_incidencia > 0)
            {
                if (client.UpdateIncidencia(IncidenciaModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Incidència actualitzada correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error actualitzant l'Incidència");
                }
            }
            NotifyOfPropertyChange(LastError);
            NotifyOfPropertyChange(FormErrors);
        }

        public void Delete()
        {
            if (Id > 0)
            {
                if (client.DeleteIncidencia(Id, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Incidència eliminada correctament");
                    // Si l'esborrat ha sigut correcte s'obre un nou formulari.
                    New();
                }
                else
                {
                    MessageBox.Show("Error eliminant l'Incidència");
                }
            }
        }

        public void Cancel()
        {
            New();
        }

        // Afegeix l'id de l'Element d'Enllumenat al Caché de l'aplicació al formulari.
        public void AddElement()
        {
            if (CacheMain.CurrentElement != null && CacheMain.CurrentElement.Id > 0)
            {
                IdElement = CacheMain.CurrentElement.Id;
            }
            else
            {
                MessageBox.Show("Element no trobat a la memòria.");
            }
        }

        # endregion

        # region Events

        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("DeleteIncidencia"))
            {
                Delete();
            }
        }

        # endregion

        #region IDataErrorInfo Members

        private string _dataError = string.Empty;

        public string Error
        {
            get { return _dataError; }
        }

        protected Dictionary<string, string> _dataErrors =
                       new Dictionary<string, string>();

        public string this[string columnName]
        {
            get
            {
                //Activa o desactiva el missatge general d'errors
                if (_dataErrors.Count != 0)
                {
                    FormErrors = "Visible";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                else
                {
                    FormErrors = "Collapsed";
                    NotifyOfPropertyChange(() => FormErrors);
                }

                if (_dataErrors.ContainsKey(columnName))
                {
                    return _dataErrors[columnName];
                }

                else
                    return null;
            }
        }

        #endregion
    }
}
