﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Ordre
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Linq;
    using System.Windows;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(OrdreViewModel))]
    public class OrdreViewModel : Screen, IDataErrorInfo, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _formErrors;
        private string _lastError = "Formulari incomplert o amb errors";
        
        private List<EstatOrdreDTO> _estatsList;
        private List<TipusPrioritatDTO> _prioritatsList;        
        private List<UsuariDTO> _tecnicsList;
        private int _idAddRemoveIncidencia;
        private ObservableCollection<IncidenciaDTO> _incidenciesObservable;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private OrdreDTO _ordreModel;

        [ImportingConstructor]
        public OrdreViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            EstatsList = client.GetAllEstatsOrdre();
            PrioritatsList = client.GetAllTipusPrioritat();
            TecnicsList = client.GetAllTecnics();

            IncidenciesObservable = new ObservableCollection<IncidenciaDTO>();
        }

        # region VIEW

        // Accions a efectuar quan la vista ha estat enllaçada al ViewModel
        protected override void OnViewAttached(object view, object context)
        {
            base.OnViewAttached(view, context);

            // Comprova si s'ha de fer servir el caché del model o crear un nou Model.
            if (CacheMain.IsNewOrdre)
            {
                OrdreModel = new OrdreDTO()
                {
                    Id_ordreReparacio = 0,
                    DataCreacio = DateTime.Now,
                    DataResolucio = DateTime.Now.AddDays(3),
                    Id_supervisor = CacheMain.CurrentUserId,
                    ComentariSupervisor = "Sense comentaris",
                    ComentariTecnic = "Sense comentaris",
                    IncidenciesList = new List<IncidenciaDTO>()
                };

                CacheMain.IsNewOrdre = false;
                CacheMain.CurrentOrdre = OrdreModel;
            }
            else
            {
                OrdreModel = CacheMain.CurrentOrdre;
                // Copia la llista d'incidèncias en la ObservableCollection del ViewModel
                IncidenciesObservable = new ObservableCollection<IncidenciaDTO>(OrdreModel.IncidenciesList);
            }
        }

        // Accions a efectuar quan es surt de la vista
        protected override void OnDeactivate(bool close)
        {
            CacheMain.CurrentOrdre = OrdreModel;
            base.OnDeactivate(close);
        }

        # endregion

        # region Error
        public string FormErrors
        {
            get { return _formErrors; }
            set
            {
                _formErrors = value;
                NotifyOfPropertyChange(() => FormErrors);
                NotifyOfPropertyChange(() => CanSave);
            }
        }

        public string LastError
        {
            get { return _lastError; }
            set
            {
                _lastError = value;
                NotifyOfPropertyChange(() => LastError);
            }
        }
        # endregion

        # region Properties

        public OrdreDTO OrdreModel
        {
            get { return _ordreModel; }
            set
            {
                _ordreModel = value;
                NotifyOfPropertyChange(() => OrdreModel);
                FormErrors = "Collapsed";
                NotifyOfPropertyChange(string.Empty);
            }
        }

        public List<TipusPrioritatDTO> PrioritatsList
        {
            get
            {
                return _prioritatsList;
            }
            set
            {
                _prioritatsList = value;
                NotifyOfPropertyChange(() => PrioritatsList);
            }
        }

        public List<EstatOrdreDTO> EstatsList
        {
            get
            {
                return _estatsList;
            }
            set
            {
                _estatsList = value;
                NotifyOfPropertyChange(() => EstatsList);
            }
        }

        public List<UsuariDTO> TecnicsList
        {
            get
            {
                return _tecnicsList;
            }
            set
            {
                _tecnicsList = value;
                NotifyOfPropertyChange(() => TecnicsList);
            }
        }

        /// <summary>
        /// Identificació de l'Ordre a la BBDD
        /// </summary>
        public int Id
        {
            get { return OrdreModel.Id_ordreReparacio; }
            set
            {
                OrdreModel.Id_ordreReparacio = value;
                NotifyOfPropertyChange(() => Id);
                NotifyOfPropertyChange(() => CanSave);

                if (value < 0)
                {
                    LastError = "L'Id és incorrecte";
                    _dataErrors["Id"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Id"))
                        _dataErrors.Remove("Id");
                }
            }
        }

        public int Tecnic
        {
            get { return OrdreModel.Id_tecnic; }
            set
            {
                OrdreModel.Id_tecnic = value;
                NotifyOfPropertyChange(() => Tecnic);
                NotifyOfPropertyChange(() => CanSave);

                if (value < 1)
                {
                    LastError = "Es requerix el Tècnic";
                    _dataErrors["Tecnic"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Tecnic"))
                        _dataErrors.Remove("Tecnic");
                }
            }
        }

        public DateTime DataCreacio
        {
            get { return OrdreModel.DataCreacio; }
            set
            {
                OrdreModel.DataCreacio = value;
                NotifyOfPropertyChange(() => DataCreacio);
                NotifyOfPropertyChange(() => CanSave);

                if (DataCreacio == null)
                {
                    LastError = "Data de creació incorrecta";
                    _dataErrors["DataCreacio"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("DataCreacio"))
                        _dataErrors.Remove("DataCreacio");
                }
            }
        }

        public DateTime DataResolucio
        {
            get { return OrdreModel.DataResolucio; }
            set
            {
                OrdreModel.DataCreacio = value;
                NotifyOfPropertyChange(() => DataResolucio);
                NotifyOfPropertyChange(() => CanSave);
            }
        }

        public string Prioritat
        {
            get { return OrdreModel.Id_tipusPrioritat; }
            set
            {
                OrdreModel.Id_tipusPrioritat = value;
                NotifyOfPropertyChange(() => Prioritat);
                NotifyOfPropertyChange(() => CanSave);

                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix la Prioritat";
                    _dataErrors["Prioritat"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Prioritat"))
                        _dataErrors.Remove("Prioritat");
                }
            }
        }

        public int Estat
        {
            get { return OrdreModel.Id_estatOrdre; }
            set
            {
                OrdreModel.Id_estatOrdre = value;
                NotifyOfPropertyChange(() => Estat);
                NotifyOfPropertyChange(() => CanSave);

                if (value < 1)
                {
                    LastError = "Es requerix l'estat";
                    _dataErrors["Estat"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Estat"))
                        _dataErrors.Remove("Estat");
                }
            }
        }

        public string ComentariSupervisor
        {
            get { return OrdreModel.ComentariSupervisor; }
            set
            {
                OrdreModel.ComentariSupervisor = value;
                NotifyOfPropertyChange(() => ComentariSupervisor);
            }
        }

        public int IdAddRemoveIncidencia
        {
            get { return _idAddRemoveIncidencia; }
            set
            {
                _idAddRemoveIncidencia = value;
                NotifyOfPropertyChange(() => IdAddRemoveIncidencia);
                NotifyOfPropertyChange(() => CanRemoveIncidencia);
            }
        }

        public ObservableCollection<IncidenciaDTO> IncidenciesObservable
        {
            get { return _incidenciesObservable; }
            set
            {
                _incidenciesObservable = value;
                NotifyOfPropertyChange(() => IncidenciesObservable);
            }
        }
       
        # endregion

        # region Commands

        public void UpdateById()
        {
            if (Id > 0)
            {
                OrdreModel = client.GetOrdreById(Id);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        public void New()
        {
            events.Publish(new CommandEvent { CommandToExecute = "ShowNewOrdre" });
        }

        public bool CanSave
        {
            get
            {
                bool isValid = (Id >= 0 && DataCreacio != null && !String.IsNullOrEmpty(Prioritat)
                                && Estat > 0 && IncidenciesObservable != null && IncidenciesObservable.Count > 0);
                return isValid;
            }
        }

        public void Save()
        {                       
            if (OrdreModel.Id_ordreReparacio == 0)
            {
                if (client.AddOrdre(OrdreModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Ordre de Reparació enregistrada correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error enregistrant l'Ordre de Reparació");
                }
            }
            else if (OrdreModel.Id_ordreReparacio > 0)
            {
                if (client.UpdateOrdre(OrdreModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Ordre de Reparació actualitzada correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error actualitzant l'Ordre de Reparació");
                }
            }
        }

        public void AddIncidencia()
        {
            // Si no hi ha un valor d'ID d'incidència al textbox s'agafa de la memòria caché.
            if (IdAddRemoveIncidencia == 0)
            {
                if (CacheMain.CurrentIncidencia != null && CacheMain.CurrentIncidencia.Id_incidencia > 0)
                {
                    IdAddRemoveIncidencia = CacheMain.CurrentIncidencia.Id_incidencia;
                }
                else
                {
                    MessageBox.Show("Incidència no trobada");
                    return;
                }
            }

            // Si hi ha un valor d'ID d'incidència al textbox

            // Comprova que la Incidència que es vol afegir és a la Base de Dades
            IncidenciaDTO addedIncidencia = client.GetIncidenciaById(IdAddRemoveIncidencia);
            if (addedIncidencia.Id_incidencia != 0)
            {
                // Comprova que la Incidència no existeix ja a la colecció.
                IncidenciaDTO existingIncidencia = (from i in IncidenciesObservable
                                                    where i.Id_incidencia == addedIncidencia.Id_incidencia
                                                    select i).SingleOrDefault();
                if (existingIncidencia == null)
                {
                    IncidenciesObservable.Add(addedIncidencia);
                    OrdreModel.IncidenciesList.Add(addedIncidencia);
                    
                    IdAddRemoveIncidencia = 0;
                    NotifyOfPropertyChange(() => IdAddRemoveIncidencia);                   
                    NotifyOfPropertyChange(() => CanSave);
                }
                else 
                {
                    MessageBox.Show("La Incidència ja està afegida a l'Ordre");
                    IdAddRemoveIncidencia = 0;
                    NotifyOfPropertyChange(() => IdAddRemoveIncidencia);
                    NotifyOfPropertyChange(() => CanRemoveIncidencia);
                }
            }
            else
            {
                MessageBox.Show("Incidència no trobada al Sistema");
                IdAddRemoveIncidencia = 0;
                NotifyOfPropertyChange(() => IdAddRemoveIncidencia);
                NotifyOfPropertyChange(() => CanRemoveIncidencia);
            }
        }

        public bool CanRemoveIncidencia
        {
            get
            {
                bool isValid = (IdAddRemoveIncidencia > 0);
                return isValid;
            }
        }

        public void RemoveIncidencia()
        {
            // Comprova que la Incidència que es vol esborrar és a la llista de l'Ordre.
            IncidenciaDTO removedIncidencia = (from inc in IncidenciesObservable
                                                where inc.Id_incidencia == IdAddRemoveIncidencia
                                                select inc).SingleOrDefault();

            if ( removedIncidencia != null)
            {
                IncidenciesObservable.Remove(removedIncidencia);
                OrdreModel.IncidenciesList.Remove(removedIncidencia);
            }
        }

        public void Delete()
        {
            if (Id > 0)
            {
                if (client.DeleteOrdre(Id, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Ordre de Reparació eliminada correctament");
                    // Si l'esborrat ha sigut correcte s'obre un nou formulari.
                    New();
                }
                else
                {
                    MessageBox.Show("Error eliminant l'Ordre de Reparació");
                }
            }
        }

        public void Cancel()
        {
            New();
        }

        // Afegeix l'id del Tècnic al Caché de l'aplicació al formulari.
        public void AddTecnic()
        {
            if (CacheMain.CurrentTecnic != null && CacheMain.CurrentTecnic.Id > 0)
            {
                Tecnic = CacheMain.CurrentTecnic.Id;
            }
            else
            {
                MessageBox.Show("Tècnic no trobat a la memòria.");
            }
        }

        # endregion

        #region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("DeleteOrdre"))
            {
                Delete();
            }
        }
        # endregion

        #region IDataErrorInfo Members

        private string _dataError = string.Empty;

        public string Error
        {
            get { return _dataError; }
        }

        protected Dictionary<string, string> _dataErrors =
                       new Dictionary<string, string>();

        public string this[string columnName]
        {
            get
            {
                //Activa o desactiva el missatge general d'errors
                if (_dataErrors.Count != 0)
                {
                    FormErrors = "Visible";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                else
                {
                    FormErrors = "Collapsed";
                    NotifyOfPropertyChange(() => FormErrors);
                }

                if (_dataErrors.ContainsKey(columnName))
                {
                    return _dataErrors[columnName];
                }

                else
                    return null;
            }
        }

        #endregion
    }
}


